const yargs = require('yargs')
const tpapiManager = require('tpapi');
const propertiesReader = require('properties-reader');
const winston = require('winston');

const argv = yargs
	.option('apiUrl', {
        alias: 'a',
        description: 'TPAPI URL',
		type: 'text'
    })
	.option('apiUser', {
        description: 'User to login to TPAPI',
        type: 'text'
    })
	.option('apiPassword', {
        description: 'Password to login to TPAPI',
        type: 'text'
    })
	.option('configFile', {
        description: 'A configuration file',
		default: 'config/application.properties',
        type: 'text'
    })
	.option('stompDebug', {
        description: 'Debug STOMP connect',
		type: 'text'
    })
	.option('logLevel', {
        description: 'Log level to output',
		type: 'text'
    })
	
    .help()
    .alias('help', 'h')
    .argv;

var properties = propertiesReader(argv.configFile);

if(argv.apiUrl) properties.set('tpapi.url',argv.apiUrl);
if(argv.apiUser) properties.set('tpapi.user',argv.apiUser);
if(argv.apiPassword) properties.set('tpapi.password',argv.apiPassword);
if(argv.logLevel) properties.set('tpapi.logLevel',argv.logLevel);
if(argv.stompDebug) properties.set('tpapi.stompDebug',argv.logLevel);

const logger = winston.createLogger({
	
	transports: [
		new winston.transports.Console({
			level: properties.get('tpapi.logLevel'),	
		})
	],
	format: winston.format.combine(
		winston.format.timestamp(),
		winston.format.printf(info => {
			return `${info.timestamp} [${info.level}] : ${JSON.stringify(info.message)}`;
		}),
	),
});

tpapi = new tpapiManager(
	properties.get('tpapi.url'), 
	properties.get('tpapi.user'), 
	Buffer.from(properties.get('tpapi.password')).toString('base64')
);

tpapi.getToken()
.then(() => { return tpapi.getUserAttributes() })
.then((response) => {
	companyId = response.companyId;
	logger.info("Got TPAPI Token");
	return tpapi.connectToStomp("/topic/org." + companyId + ".tickets.#", true, messageCallback, errorCallback, properties.get('tpapi.stompDebug'));
})
.catch(error => {
	logger.error("Could not get Company Information");
	logger.error(error);
});

function errorCallback(error){
	logger.error('STOMP Error: ');
    logger.error(error);
}

function messageCallback(message) {
	logger.info('Full Message: ' + message);
	logger.info('Message Body: ' + message.body);
	
	body = JSON.parse(message.body);
	
	switch(body.eventType){
		case 'tickets.updated':
			ticketUpdated(body.eventId.split('.')[2]);
			break;
		case 'tickets.created':
			ticketCreated(body.eventId.split('.')[2]);
			break;
		default:
			logger.info(body.eventId);
	}
}

function ticketUpdated(id){
	logger.info('Ticket ' + id + ' was updated!');
}

function ticketCreated(id){
	logger.info('Ticket ' + id + ' was created!');
}